/** \file rtaixml_master.cpp
*COPYRIGHT (C) 2006  Michele Basso   (basso@dsi.unifi.it)
*                    Marco Romagnoli (romagnoli@control.dsi.unifi.it)
*                    Massimo Vassalli(vassalli@inoa.it)

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
*/

#include "rtaixml_master.h"

#include<iostream>
#include<string>
#include<sstream>

using namespace XmlRpc;
//using namespace rtaixml_domain;
namespace rtaixml_domain {

XmlRpcServer xMaster;
RTDomain *Real_Time_Domain;
static std::string slave_script("rslave");
static std::string slave_directory("./");


/*******************************************************************************************************
*  Accessory function
*******************************************************************************************************/

/** \brief Convert int to string.
* 
* @param i: int to be converted.
* @return converted string.
*/ 
std::string itos(int i)
{
	std::stringstream s;
	s << i;
	return s.str();
}

/*******************************************************************************************************
*  End of Accessory function
*******************************************************************************************************/

/** @defgroup master Master Server (rtmanager)
*  This group contains all the Remote Procedure Call available via XMLRPC to deal with the Master Server.
*  @{
*/

/*******************************************************************************************************
*  XmlRpcServerMethod Class Definition
*******************************************************************************************************/

/** \class Connection_Request
* \brief Let you request a connection to the rtaixml domain.
* 
* This method is available as web service. It allows you to request a connection with
* the specified target. This operation is necessary before doing any other action on the target.
* It is the first step in connecting to a target.
* <div class="fragment"><div class="fragment1">
* <span class="preprocessor"><b>IMPORTANT! READ CAREFULLY!</b></span><br>
* <span class="">The XMLRPC method Connection_Request is located in rtaixml.cpp and in 
* rtaixml_master.cpp because the framework can operate in master/slave mode and in standalone mode.
* The methods have the same interface but work differently.</span><br>
* <span class="keyword">For more info see page \ref master_slave. </span><br>
* <span class="keyword">See also the class Connection_Request_. </span><br>
* </div></div>
* @param params it has to contain as element 0 the target name to connect. The real name of the 
* mailbox is determined in the slave server that manages the real connectionand it 
* is obtained by calling the function get_an_id, that add at the end of the name a 
* progressive integer that it is used to distinguish mailboxes of different signals
* @param result This variable contains the result of the execution of this 
* function. It contains three values. The first "id_session" is an 
* string that represents the session identification necessary to be able to connect to
* to the slave server. The second "state" is an integer value (0 everything OK - 1 some errors).
* The third "str" contains a human readable string explaining what happens. The last parameter
* is present only if "state" is 0 and contains the XML port where the slave server is listening.
* 
* @return see parameter result. 
*/
class Connection_Request : public XmlRpcServerMethod
{
public:
	Connection_Request(XmlRpcServer* xMaster) : XmlRpcServerMethod("Connection_Request", xMaster) {}

	void execute(XmlRpcValue& params, XmlRpcValue& result)
	{
		bool bMastership=false;
		
		if(IS_DEBUG) std::cout << std::endl << "--------------------------------------------------------------------------------" << std::endl;
		if(IS_DEBUG) std::cout << ">> Connection_Request <<" << std::endl;
		if(IS_DEBUG) std::cout << "--------------------------------------------------------------------------------" << std::endl;
		XmlRpcValue status;

		Real_Time_Domain->PrintCurrentStatus();

		pid_t childpid; 
		std::string sTargetName;

		if(IS_DEBUG) std::cout << "## Checking parameters..." << std::endl;
		
		// 0. Formal check
		if(IS_DEBUG) std::cout << "** Formality...";
		if(!params.valid() || params.size()<1) {
			
			if(IS_DEBUG) std::cout << "FAILURE." << std::endl; 		
			status["id_session"]= "";
			status["state"]=(int)1;
			status["str"]="Number of parameters in calling CONNECTION_REQUEST incorrect";
			result=status;
			if(IS_DEBUG) std::cout << ">> End of Connection_Request <<" << std::endl;
			if(IS_DEBUG) std::cout << "--------------------------------------------------------------------------------" << std::endl;
			return;
		}
		if(IS_DEBUG) std::cout << "OK. " << params.size() << " parameter(s) received." << std::endl; 		
		
		// 2. Clear domain table 
		if(IS_DEBUG) { std::cout << "** Clearing..."; std::fflush(stdout);}
		int iWaitingClientSlaves = Real_Time_Domain->ClearDomain();
		if(iWaitingClientSlaves>0) {
			if(IS_DEBUG) std::cout  << "Found " << iWaitingClientSlaves << " slave(s) without target enabled waiting for a client interaction." << std::endl; 		
		}
		if(IS_DEBUG) std::cout << "OK." << std::endl; 		
		
		// 2a. Check Mastership
		if(params.size()>1) {
			if(std::string(params[1]).compare(Real_Time_Domain->getMasterName())) {
				if(IS_DEBUG) std::cout  << "FAILURE -- " << "Master Name provided: "<< params[1] << " - MasterName correct: " << Real_Time_Domain->getMasterName() << std::endl; 		
				bMastership=false;
				if(IS_DEBUG) std::cout << "Mastership NOT taken!" << std::endl; 		
			} else {
				bMastership=true;
				if(IS_DEBUG) std::cout << "Mastership taken." << std::endl; 		
			}
		}

		// 3. Secure policy check
		if(IS_DEBUG) std::cout << "** Security...";
		std::fflush(stdout);
		if(!bMastership && Real_Time_Domain->SecureCheck()==false)
		{
			if(IS_DEBUG) std::cout << "FAILURE." << std::endl; 		
			status["id_session"]= "";
			status["state"]=(int)1;
			status["str"]="Secure policy not passed.";
			result=status;
			if(IS_DEBUG) std::cout << ">> End of Connection_Request <<" << std::endl;
			if(IS_DEBUG) std::cout << "--------------------------------------------------------------------------------" << std::endl;
			return;
		}
		if(IS_DEBUG) std::cout << "OK." << std::endl; 		

		sTargetName=std::string(params[0]);

		if(IS_DEBUG) std::cout << "-----" << std::endl; 		
		if(IS_DEBUG) std::cout << "Target Name: " << sTargetName << std::endl; 		
		if(IS_DEBUG) std::cout << "-----" << std::endl; 		
		
		// 5. Attach the slave server and the related target to the list
		if(IS_DEBUG) std::cout << "** Adding new (Target,Server)...";
		if(!Real_Time_Domain->AddNewActorAndChoosePort(sTargetName)) {
			if(IS_DEBUG) std::cout << "The target "<< sTargetName <<" is already managed by slave server " << Real_Time_Domain->getSlaveName(sTargetName,0)  <<"." << std::endl; 		
			if(bMastership) {
				if(IS_DEBUG) std::cout << "Taking mastership..." << std::endl;
				//Devo stoppare lo rtaixml attuale e lanciarne uno nuovo.
				if(!(Real_Time_Domain->StopRemoveSlaveServer(Real_Time_Domain->getSlaveName(sTargetName,0)) && Real_Time_Domain->AddNewActorAndChoosePort(sTargetName))) {
					status["id_session"]= "";
					status["state"]=(int)1;
					status["str"]="Mastership problems.";
					result=status;
					if(IS_DEBUG) std::cout << ">> End of Connection_Request <<" << std::endl;
					if(IS_DEBUG) std::cout << "--------------------------------------------------------------------------------" << std::endl;
					return;
				}
			} else {
				status["id_session"]= "";
				status["state"]=(int)1;
				status["str"]="Multiple policy not passed.";
				result=status;
				if(IS_DEBUG) std::cout << ">> End of Connection_Request <<" << std::endl;
				if(IS_DEBUG) std::cout << "--------------------------------------------------------------------------------" << std::endl;
				return;
			}
			if(IS_DEBUG) std::cout << "Mastership taken!" << std::endl;
		}
			
		
		if(IS_DEBUG) std::cout << "done." << std::endl;
		
		childpid = fork();
		if(IS_DEBUG) std::cout << "** Starting the slave server..." ; 		
		
		if (childpid < 0)
		{
			if(IS_DEBUG) std::cout << "Fork() error." << std::endl; 		
			perror("fork"); 
			status["id_session"]= "";
			status["state"]=(int)1;
			status["str"]="Generic error.";
			result=status;
		}
		if (childpid == 0)
		{
			xMaster.shutdown();
			if(IS_DEBUG) std::cout << "child: executing slave script." << std::endl; 		
            std::cout << "Executing 'rtaixml_slave'...." << std::endl;
	        std::string sAppo = slave_directory + slave_script;
	        execl(sAppo.c_str(),slave_script.c_str(), 
					sTargetName.c_str(),
					itos(Real_Time_Domain->getPortXMLSlave(sTargetName,0)).c_str(),
					(Real_Time_Domain->getMasterName()).c_str(),
					itos(Real_Time_Domain->getPortXML()).c_str(),	        		
					Real_Time_Domain->getSessionId(sTargetName).c_str(),
					itos(Real_Time_Domain->getPortXMLSlave(sTargetName,0)+1).c_str(),
					(IS_DEBUG?"-v":""),0);
				std::cout << "...done." << std::endl;
				exit(0); 
		}
		else 
		{
			wait(NULL);
			if(IS_DEBUG) std::cout << "father: child terminated." << std::endl; 		
         //   sleep(1);
				std::cout << "slave server '"<< Real_Time_Domain->getSlaveName(sTargetName,0) <<"' started to manage " << sTargetName <<" target.\n"; 
		}
		if(IS_DEBUG) std::cout << "...done." << std::endl;

		if(IS_DEBUG) std::cout << "-----" << std::endl; 		
		if(IS_DEBUG) std::cout << "Slave Name: " << Real_Time_Domain->getSlaveName(sTargetName,0) << std::endl; 		
		if(IS_DEBUG) std::cout << "Session ID: " << Real_Time_Domain->getSessionId(sTargetName) << std::endl; 		
		if(IS_DEBUG) std::cout << "-----" << std::endl; 		

		// 6. return values setting. id_session and port necessary to contact the slave server.
		status["id_session"] = Real_Time_Domain->getSessionId(sTargetName);
		status["state"]      = (int)0;
		status["str"]        = "Connection accepted. Session attached.";
		status["port"]       = Real_Time_Domain->getPortXMLSlave(sTargetName,0);
		result=status;

		Real_Time_Domain->PrintCurrentStatus();

		if(IS_DEBUG) { std::cout << ">> End of Connection_Request <<" << std::endl; std::fflush(stdout); }
		if(IS_DEBUG) std::cout << "--------------------------------------------------------------------------------" << std::endl;
	}
} connection_request(&xMaster);


/** \class Stop_Master_Server
* \brief It allows you to stop the master server via XMLRPC.
* 
* This method is available as web service. It allows you to stop the master server.
* 
* @param params it has to contain as element 0 the master server name. When 
* executing the server is possible to provide a name (-n option).
* This name is used to keep a level of security. In order to stop the server you have
* to know its name, in this way a little of security is provided.
* @param result This variable contains the result of the execution of this 
* function. It contains two values. The first "str" is an human readable
* string that explains what's happened. The second "state" is an integer value 
* (0 everything OK - 1 some errors).
*  
* @return see parameter result. 
*/
class Stop_Master_Server : public XmlRpcServerMethod
{
public:
Stop_Master_Server(XmlRpcServer* xMaster) : XmlRpcServerMethod("Stop_Master_Server", xMaster) {}

void execute(XmlRpcValue& params, XmlRpcValue& result)
{
	if(IS_DEBUG) std::cout << std::endl << "--------------------------------------------------------------------------------" << std::endl;
	if(IS_DEBUG) std::cout << ">> Stop_Master_Server <<" << std::endl;
	if(IS_DEBUG) std::cout << "--------------------------------------------------------------------------------" << std::endl;
	XmlRpcValue status;
	if(IS_DEBUG) std::cout << "## Checking parameters..." << std::endl;
	
	if(IS_DEBUG) std::cout << "** Formality...";
	if(!params.valid() || params.size()!=1) {
		if(IS_DEBUG) std::cout << "FAILURE." << std::endl; 		
		status["str"]="Number of parameters in calling STOP_MASTER_SERVER incorrect";
		status["state"]=(int)1;
		result=status;
		if(IS_DEBUG) std::cout << ">> End of Stop_Master_Server <<" << std::endl;
		if(IS_DEBUG) std::cout << "--------------------------------------------------------------------------------" << std::endl;
		return;
	}
	if(IS_DEBUG) std::cout << "OK. " << params.size() << " parameter(s) received." << std::endl; 		

	// 1. legality check
	if(IS_DEBUG) std::cout << "** Legality...";
	if(std::string(params[0]).compare(Real_Time_Domain->getMasterName())) {
		if(IS_DEBUG) std::cout  << "FAILURE -- " << "Master Name provided: "<< params[0] << " - MasterName correct: " << Real_Time_Domain->getMasterName() << std::endl; 		
		status["str"]="Servername incorrect.";
		status["state"]=(int)1;
		result=status;
		if(IS_DEBUG) std::cout << ">> End of Stop_Master_Server <<" << std::endl;
		if(IS_DEBUG) std::cout << "--------------------------------------------------------------------------------" << std::endl;
		return;
	}
	if(IS_DEBUG) std::cout << "OK. MasterName: " << Real_Time_Domain->getMasterName() << std::endl; 		

	Real_Time_Domain->PrintCurrentStatus();

	if(IS_DEBUG) std::cout << "** Closing all active slave servers..." << std::endl;
	XmlRpcValue oneArgs, resultslave;
	std::vector< SlaveInfo > vsiAllSlaveInfo = Real_Time_Domain->getAllSlaveInfo();
	if(IS_DEBUG) std::cout << "** Number of slave servers: " << vsiAllSlaveInfo.size() << std::endl;
	for(unsigned int i=0;i<vsiAllSlaveInfo.size();i++) {
		if(IS_DEBUG) std::cout << "** \t Slave server name: " << vsiAllSlaveInfo.at(i).getName() << std::endl;
			oneArgs[0]=vsiAllSlaveInfo.at(i).getName().c_str();
			//We should disable tha capability of the slave that on closing make an XmlRpc call on the master...we will have stall.
			oneArgs[1]="master";
			if((new XmlRpcClient(vsiAllSlaveInfo.at(i).getIP().c_str(), vsiAllSlaveInfo.at(i).getPortXML()))->execute("Stop_Server", oneArgs, resultslave))
				if(IS_DEBUG) std::cout << resultslave << "\n";
			else
				if(IS_DEBUG) std::cout << "Slave Server "<< vsiAllSlaveInfo.at(i).getName() << " not found on port " << vsiAllSlaveInfo.at(i).getPortXML() << ".\n";
				
	}
	if(IS_DEBUG) std::cout << "Closed " << vsiAllSlaveInfo.size() << " slave servers!" << std::endl;
	if(IS_DEBUG) std::cout << "...done" << std::endl;

	if(IS_DEBUG) std::cout << "** XMLRPC server shutdown...";
	xMaster.shutdown();	//Stop of the XMLRPC Server
	if(IS_DEBUG) std::cout << "OK" << std::endl;

	status["str"]="Master Server stopped.";
	status["state"]=(int)0;
	result=status;


	Real_Time_Domain->PrintCurrentStatus();
	if(IS_DEBUG) std::cout << "** Delete Real Time Domain...";
	delete Real_Time_Domain;
	if(IS_DEBUG) std::cout << "OK" << std::endl;
	
	if(IS_DEBUG) std::cout << ">> End of Stop_Master_Server <<" << std::endl;
	if(IS_DEBUG) std::cout << "--------------------------------------------------------------------------------" << std::endl;
}

std::string help() { return std::string("Allows Master Server stop"); }

} stop_master_server(&xMaster);    // This constructor registers the method with the server


/** \class Close_Session
* \brief It allows you to detach a session from the rtaixml domain via XMLRPC.
* 
* This method is available as web service. It allows you to detach a session.
* 
* @param params it has to contain as element 0 the master server name. 
* As element 1 the id of the session you want to detach.
* @param result This variable contains the result of the execution of this 
* function. It contains two values. The first "str" is an human readable
* string that explains what's happened. The second "state" is an integer value 
* (0 everything OK - 1 some errors).
*  
* @return see parameter result. 
*/
class Close_Session : public XmlRpcServerMethod
{
public:
Close_Session(XmlRpcServer* xMaster) : XmlRpcServerMethod("Close_Session", xMaster) {}

void execute(XmlRpcValue& params, XmlRpcValue& result)
{
	if(IS_DEBUG) std::cout << std::endl << "--------------------------------------------------------------------------------" << std::endl;
	if(IS_DEBUG) std::cout << ">> Close_Session <<" << std::endl;
	if(IS_DEBUG) std::cout << "--------------------------------------------------------------------------------" << std::endl;
	XmlRpcValue status;
	if(IS_DEBUG) std::cout << "## Checking parameters..." << std::endl;
	
	if(IS_DEBUG) std::cout << "** Formality...";
	//0 masterName 1 sessionid
	if(!params.valid() || params.size()!=2) {
		if(IS_DEBUG) std::cout << "FAILURE." << std::endl; 		
		status["str"]="Number of parameters in calling Close_Session incorrect";
		status["state"]=(int)1;
		result=status;
		if(IS_DEBUG) std::cout << ">> End of Close_Session <<" << std::endl;
		if(IS_DEBUG) std::cout << "--------------------------------------------------------------------------------" << std::endl;
		return;
	}
	if(IS_DEBUG) std::cout << "OK. " << params.size() << " parameter(s) received." << std::endl; 		

	// 1. legality check
	if(IS_DEBUG) std::cout << "** Legality...";
	if(std::string(params[0]).compare(Real_Time_Domain->getMasterName())) {
		if(IS_DEBUG) std::cout  << "FAILURE -- " << "Master Name provided: "<< params[0] << " - MasterName correct: " << Real_Time_Domain->getMasterName() << std::endl; 		
		status["str"]="Servername incorrect.";
		status["state"]=(int)1;
		result=status;
		if(IS_DEBUG) std::cout << ">> End of Close_Session <<" << std::endl;
		if(IS_DEBUG) std::cout << "--------------------------------------------------------------------------------" << std::endl;
		return;
	}
	if(IS_DEBUG) std::cout << "OK. MasterName: " << Real_Time_Domain->getMasterName() << std::endl; 		

	Real_Time_Domain->PrintCurrentStatus();

	if(IS_DEBUG) std::cout << "** Removing (Target,Server)...";
	if(Real_Time_Domain->RemoveActor(Real_Time_Domain->getTaskName(params[1])))
	{
		if(IS_DEBUG) std::cout << "OK." << std::endl; 		
		status["str"]="Session detached";
		status["state"]=(int)0;
	} else {
		if(IS_DEBUG) std::cout << "FAILURE. Session " << Real_Time_Domain->getTaskName(params[1]) << "does not exist."<< std::endl; 		
		status["str"]="Session NOT exists";
		status["state"]=(int)1;
	}

	Real_Time_Domain->PrintCurrentStatus();

	result=status;
	if(IS_DEBUG) std::cout << ">> End of Close_Session <<" << std::endl;
	if(IS_DEBUG) std::cout << "--------------------------------------------------------------------------------" << std::endl;
}

std::string help() { return std::string("Allows to Close a Session and to detach all the information from the rtaixml domain."); }

} close_session(&xMaster);    // This constructor registers the method with the server


/** \class Info_RT
* \brief It allows you to get information about the rtaixml domain via XMLRPC.
* 
* This method is available as web service. It allows you to get info. (TBD)
* Maybe it could be used from an administration client panel to monitor the
* evolution of the rtaixml domain. It should be developed jointly with a
* statistic and advanced information tool for the rtaixml domain.
* 
* @param params it has to contain as element 0 the master server name. 
* 
* @param result This variable contains the result of the execution of this 
* function. It contains two values. The first "str" is an human readable
* string that explains what's happened. The second "state" is an integer value 
* (0 everything OK - 1 some errors).
*  
* @return see parameter result. 
*/
class Info_RT : public XmlRpcServerMethod
{
public:
Info_RT(XmlRpcServer* xMaster) : XmlRpcServerMethod("Info_RT", xMaster) {}

void execute(XmlRpcValue& params, XmlRpcValue& result)
{
	if(IS_DEBUG) std::cout << std::endl << "--------------------------------------------------------------------------------" << std::endl;
	if(IS_DEBUG) std::cout << ">> Info_RT <<" << std::endl;
	if(IS_DEBUG) std::cout << "--------------------------------------------------------------------------------" << std::endl;
	XmlRpcValue status;
	if(IS_DEBUG) std::cout << "## Checking parameters..." << std::endl;
	
	if(IS_DEBUG) std::cout << "** Formality...";
	//0 masterName 1 sessionid
	if(!params.valid() || params.size()!=1) {
		if(IS_DEBUG) std::cout << "FAILURE." << std::endl; 		
		status["str"]="Number of parameters in calling Info_RT incorrect";
		status["state"]=(int)1;
		result=status;
		if(IS_DEBUG) std::cout << ">> End of Info_RT <<" << std::endl;
		if(IS_DEBUG) std::cout << "--------------------------------------------------------------------------------" << std::endl;
		return;
	}
	if(IS_DEBUG) std::cout << "OK. " << params.size() << " parameter(s) received." << std::endl; 		

	// 1. legality check
	if(IS_DEBUG) std::cout << "** Legality...";
	if(std::string(params[0]).compare(Real_Time_Domain->getMasterName())) {
		if(IS_DEBUG) std::cout  << "FAILURE -- " << "Master Name provided: "<< params[0] << " - MasterName correct: " << Real_Time_Domain->getMasterName() << std::endl; 		
		status["str"]="Servername incorrect.";
		status["state"]=(int)1;
		result=status;
		if(IS_DEBUG) std::cout << ">> End of Info_RT <<" << std::endl;
		if(IS_DEBUG) std::cout << "--------------------------------------------------------------------------------" << std::endl;
		return;
	}
	if(IS_DEBUG) std::cout << "OK. MasterName: " << Real_Time_Domain->getMasterName() << std::endl; 		


	//IT WILL BE IMPLEMENTED AT THE RIGHT TIME!!!

	status["str"]="Info_RT";
	status["state"]=(int)0;
	result=status;
	if(IS_DEBUG) std::cout << ">> End of Info_RT <<" << std::endl;
	if(IS_DEBUG) std::cout << "--------------------------------------------------------------------------------" << std::endl;
}

std::string help() { return std::string("Allows to get information about the real time domain"); }

} info_rt(&xMaster);    // This constructor registers the method with the server

/** @} */ // end of master

/*******************************************************************************************************
* 	FINE Definizioni delle Classi XmlRpcServerMethod
*******************************************************************************************************/

struct option options[] = {
	{ "help",       	0, 0, 'h' },
	{ "verbose",    	0, 0, 'v' },
	{ "version",    	0, 0, 'V' },
	{ "servername",	1, 0, 'n' },
	{ "stop",    	0, 0, 's' },
	{ "portXML", 	1, 0, 'X' },
	{ "maxportXML", 	1, 0, 'M' },
};

void print_usage(void)
{
	fputs(
("\nUsage:  rtaixml_master [OPTIONS]\n"
"\n"
"OPTIONS:\n"
"  -h, --help\n"
"      print usage\n"
"  -v, --verbose\n"
"      verbose output\n"
"  -V, --version\n"
"      print version\n"
"  -M, --servername <MASTER SERVER NAME>\n"
"      set the server name in the rtaixml domain\n"
"  -s, --stop\n"
"      stop server on localhost on the specified port\n"
"  -X <XMLRPC_port>, --portXML <XMLRPC_port>\n"
"      number of the XMLRPC server port\n"
"  -P <MAX_XMLRPC_port>, --maxportXML <MAXXMLRPC_port>\n"
"      max number of the XMLRPC server port\n"
"\n")
		,stderr);
	exit(0);
}

} // end namespace rtaixml_domain

using namespace rtaixml_domain;

int main(int argc, char* argv[])
{
	int c, option_index = 0;
	int test;
	XmlRpcValue oneArgs, result;
	int verbosityXMLRPC=0;
	bool toStop=false;
	
	Real_Time_Domain = new RTDomainMEM();
	
	while (1) {
		c = getopt_long(argc, argv, "htn:vVsX:M:", options, &option_index);
		if (c == -1)
				break;
		switch (c) {
			case 'M':
				Real_Time_Domain->setMasterName(std::string(optarg));
				std::cout<<"Master Server name set to "<<Real_Time_Domain->getMasterName()<<"\n";
				break;
			case 's':
				toStop=true;
				break;
			case 'v':
				IS_DEBUG = true;
				verbosityXMLRPC=1;
				break;
			case 'V':
				std::cout << "rtaixml master version " << RTAIXML_MASTER_VERSION << "\nrtaixml slave version " << RTAIXML_VERSION << std::endl;
				exit(0);
			case 'X':
				test = atoi(optarg);
				if (test>1024 && test<65537)
					Real_Time_Domain->setBasePortXML(test);
				else
					std::cout << "The given port number isn't correct, using default one.\n";
				
				Real_Time_Domain->setBasePortXML(test);
				std::cout << "XMLRPC Master Port set to " << Real_Time_Domain->getPortXML() << "." << std::endl;
				break;
			case 'P':
				test = atoi(optarg);
				if (test>1024 && test<65537)
					Real_Time_Domain->setMaxPortXML(test);
				else
					std::cout << "The given port number isn't correct, using default one.\n";
				
				Real_Time_Domain->setMaxPortXML(test);
				std::cout << "MAX XMLRPC Slave Port set to " << Real_Time_Domain->getMaxPortXML() << "." << std::endl;
				break;
			case 'h':
				print_usage();
				exit(0);
			default:
				break;
		}
	}

	if(toStop)
	{
		std::cout <<  "Stopping rtai-xml master server on port " << Real_Time_Domain->getPortXML() << "..." << std::endl;
		oneArgs[0]=Real_Time_Domain->getMasterName();
		if((new XmlRpcClient("localhost", Real_Time_Domain->getPortXML()))->execute("Stop_Master_Server", oneArgs, result))
			std::cout << result << std::endl;
		else
			std::cout << "Server not found on port " << Real_Time_Domain->getPortXML() << "." << std::endl;
		exit(0);
	}
	
	std::string sAppo = slave_directory + slave_script;
	std::ifstream myfile (sAppo.c_str());
	if (myfile.is_open())
		myfile.close();
	else
	{
		std::cout << "rtmanager MUST be run from the install dir! Stopped!" << std::endl;
		return 1;
	}
	

	if(IS_DEBUG) std::cout << "=======================================================================" << std::endl;
	if(IS_DEBUG) std::cout << "\t\t\tDEBUG MODE ON" << std::endl;
	if(IS_DEBUG) std::cout << "=======================================================================" << std::endl;
	if(IS_DEBUG) {std::cout << "main(): MasterName: " << Real_Time_Domain->getMasterName() << std::endl; std::fflush(stdout);}
	if(daemon(1,(IS_DEBUG?1:0))) 
	{
		std::cout << "Error in creating server process, execution stopped.\n";
		exit(1);
	}
	//XmlRpc::setVerbosity(5);

	// Create the server socket on the specified port
	xMaster.bindAndListen(Real_Time_Domain->getPortXML());

	// Enable introspection
//	xMaster.enableIntrospection(true);

	// Wait for requests indefinitely
	xMaster.work(-1.0);

	return 0;
}
